@@ -10,8 +10,8 @@ from BeautifulSoup import Comment
 from BeautifulSoup import CData
 from BeautifulSoup import NavigableString
 from BeautifulSoup import Tag
-from urlparse import urljoin
 from urlparse import urlparse
+from urlparse import urlunparse
 from urlparse import urlsplit
 from urlparse import urlunsplit
 from datetime import datetime
@@ -133,6 +133,63 @@ def quote_url_path(url):
     return urlunsplit((p[0], p[1], urllib.quote(p[2], "/%"), p[3], p[4]))
 
 
+def urljoin_rfc1808(base, url, allow_fragments=True):
+    """Join a base URL and a possibly relative URL to form an absolute
+    interpretation of the latter. RFC 1808 superceeds 2369 here."""
+
+    from urlparse import uses_relative
+    from urlparse import uses_netloc
+
+    if not base:
+        return url
+    if not url:
+        return base
+    bscheme, bnetloc, bpath, bparams, bquery, bfragment = \
+            urlparse(base, '', allow_fragments)
+    scheme, netloc, path, params, query, fragment = \
+            urlparse(url, bscheme, allow_fragments)
+    if scheme != bscheme or scheme not in uses_relative:
+        return url
+    if scheme in uses_netloc:
+        if netloc:
+            return urlunparse((scheme, netloc, path,
+                               params, query, fragment))
+        netloc = bnetloc
+    if path[:1] == '/':
+        return urlunparse((scheme, netloc, path,
+                           params, query, fragment))
+    if not path:
+        return urlunparse((scheme, netloc, bpath,
+                           params, query, fragment))
+    segments = bpath.split('/')[:-1] + path.split('/')
+    # XXX The stuff below is bogus in various ways...
+    if segments[-1] == '.':
+        segments[-1] = ''
+    while '.' in segments:
+        segments.remove('.')
+    while 1:
+        i = 1
+        n = len(segments) - 1
+        while i < n:
+            if (segments[i] == '..'
+                and segments[i-1] not in ('', '..')):
+                del segments[i-1:i+1]
+                break
+            i = i+1
+        else:
+            break
+    if segments == ['', '..']:
+        segments[-1] = ''
+    elif len(segments) >= 2 and segments[-1] == '..':
+        segments[-2:] = ['']
+    return urlunparse((scheme, netloc, '/'.join(segments),
+                       params, query, fragment))
+
+
+urljoin = urljoin_rfc1808
+#from urlparse import urljoin
+
+
 def get_or_create_tag(html, list, tag = None):
     if not tag:
         tag = html
@@ -347,9 +404,11 @@ class Item:
         self.__single_page_keywords = single_page_keywords
         self.__next_page_keywords = next_page_keywords
         self.__callback = callback
-        self.__attrs_for_fix = [("a", "href"), ("iframe", "src"), ("object", "data"), ("form", "action")]
-        self.__attrs_for_cache = \
-            [("applet", "codebase"), ("object", "codebase"), ("script", "src"), ("img", "src"), ("input", "src")]
+        self.__attrs_for_fix = [("a", "href"), ("iframe", "src"), ("form", "action")]
+        self.__attrs_for_cache = [("script", "src"), ("img", "src"), ("input", "src")]
+        self.__tags_for_removal = ["object", "applet", "embed"]
+        if not self.__allow_scripts:
+            self.__tags_for_removal.append("script")
         self.title = "untitled"
         self.__content_kwargs = content_kwargs
         self.__reformatter_url = reformatter_url
@@ -525,8 +584,7 @@ class Item:
         3. Keeping consistency by preventing extracting tags from context,
         e.g. td should always be nested into tr etc.
         """
-        if not self.__allow_scripts:
-            [script.extract() for script in self.__tag.findAll("script")]
+        [t.extract() for t in self.__tag.findAll(self.__tags_for_removal)]
         [self.__fix_urls(self.__tag.findAll(x[0]), x[1], self.__base) for x in self.__attrs_for_fix]
         [self.__cache_urls(self.__tag.findAll(x[0]), x[1], self.__base) for x in self.__attrs_for_cache]
         if self.__tag.name in ["table", "dir", "menu", "dl", "ul", "ol"]:
